body {
    font-family: Arial, sans-serif;
    line-height: 1.6;
    color: #333;
    margin: 20px;
    padding: 20px;
    background-color: #f4f4f4;
}

h1 {
    color: #444;
}

p {
    margin-bottom: 20px;
}

blockquote {
    border-left: 5px solid #ccc;
    padding-left: 15px;
    margin-left: 0;
    color: #666;
}

a {
    color: #3498db;
    text-decoration: none;
}

a:hover {
    color: #226897;
}